
from distutils.core import setup

setup(
  name = 'peekextn',
  packages_dir = {'peekextn': 'src'},
  packages = ['peekextn'],
  version = '0.3',
  description = 'Peek extension and show respective sources',
  author = 'Vadivel',
  author_email = 'vadivelmurugank@gmail.com',
  url = 'https://github.com/vadivelmurugank/peekextn',
  license = 'MIT',
)
